/***********************************************************

                     Classe CGoughArt

  Contient les methodes pour la commande articulaire
  d'une plateforme de Gough-Stewart 'symetrique'

  Auteur : N. Andreff, LASMEA, Clermont-Ferrand, France
  Date :   12 juillet 2004

************************************************************/

#include "CGoughArt.hh"
#include <iostream>
using namespace std;

#include <sys/time.h>
#include <unistd.h> // Pour le usleep
#include <math.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <unistd.h>
#include <dirent.h>
#include <string.h>
#include <sys/sysmacros.h> /* major() minor() */

#include <apci3120.h>

#include "addi_data_find_cards.c"

/*#define MAX_ANALOGINPUT_CHANNELS    16
#define  APCI3120_EOC_MODE           1
#define  APCI3120_EOS_MODE           2
#define APCI3120_ENABLE              1
#define APCI3120_DISABLE             0 
*/

// La periode d'echantillonnage en microsecondes
#define PeriodeEchantillonnage 20000

// La vitesse minimale pour qu'il y ait deplacement
#define FS 0.015

CGoughArt::CGoughArt()
{
  CGoughParamArt Param;
  for (int i=0; i<NB_VERINS; i++)
    {
      mParam[i]=Param;
#ifdef DEBUG
      cout << "Verin " << i+1 << ":" << endl;
      cout << "LongueurMin: " << mParam[i].LongueurMin << endl;      
      cout << "LongueurMax: " << mParam[i].LongueurMax<< endl;
      cout << "Lmin: " << mParam[i].Lmin<< endl;
      cout << "Lmax: " << mParam[i].Lmax<< endl;
      cout << "ZeroCodeur: " << mParam[i].ZeroCodeur<< endl;
      cout << "PasCodParMetre: " << mParam[i].PasCodParMetre<< endl;
      cout << "PasMotParMetre: " << mParam[i].PasMotParMetre<< endl;
#endif
    }

  mArretVitesse=true;
}

CGoughArt::CGoughArt(const CGoughParamArt& Param)
{

  for (int i=0; i<NB_VERINS; i++)
    {
      mParam[i]=Param;
    }

  mArretVitesse=true;
}

CGoughArt::CGoughArt(const CGoughParamArt  Param[NB_VERINS])
{

  for (int i=0; i<NB_VERINS; i++)
    {
      mParam[i]=Param[i];
    }

  mArretVitesse=true;
}




int CGoughArt::Ouvrir()
{
 
  // Initialisation de la carte
  unsigned int apci3120_card_number = 0;
  {
    apci3120_card_number = apci3120_find_cards(&mCarteAPCI3120);
    printf("total: %d cards\n",apci3120_card_number);
  }

  // Table de correspondance entree_carte[no_verin] et sortie_carte[no_verin]
  // a modifier en fonction du cablage
  entree_carte[0]=1;
  entree_carte[1]=4; 
  entree_carte[2]=5;
  entree_carte[3]=12;
  entree_carte[4]=14; 
  entree_carte[5]=9;
  
  sortie_carte[0]=0;
  sortie_carte[1]=5; 
  sortie_carte[2]=4;
  sortie_carte[3]=3;
  sortie_carte[4]=2;
  sortie_carte[5]=1;


  /*mSubdeviceLecture=0; 
  int ret = comedi_get_subdevice_type(mDevice,mSubdeviceLecture);
  if  (ret!=COMEDI_SUBD_AI)
    {
      cout << mSubdeviceLecture 
	   << " ne correspond pas a une entree analogique"
	   << endl;
      return -1;
    } 

  mSubdeviceEcriture=1; 
  ret = comedi_get_subdevice_type(mDevice,mSubdeviceEcriture);
  if  (ret!=COMEDI_SUBD_AO)
    {
      cout << mSubdeviceEcriture 
	   << " ne correspond pas a une sortie analogique"
	   << endl;

      return -1;
    } 

  str_list_Lecture.n_insns=NB_VERINS;
  str_list_Lecture.insns=str_Lecture;
  str_list_Ecriture.n_insns=NB_VERINS;
  str_list_Ecriture.insns=str_Ecriture;

  for (int i=0; i<NB_VERINS; i++)
    {
      str_Lecture[i].insn = INSN_READ;
      str_Lecture[i].n = 1;
      str_Lecture[i].subdev = mSubdeviceLecture ;
      str_Lecture[i].data = &mLectureCodeurs[i];
      str_Lecture[i].chanspec = CR_PACK( entree_carte[i],1,0);

      str_Ecriture[i].insn = INSN_WRITE;
      str_Ecriture[i].n = 1;
      str_Ecriture[i].subdev = mSubdeviceEcriture ;
      str_Ecriture[i].data = &mCommandes[i];
      str_Ecriture[i].chanspec = CR_PACK( sortie_carte[i],0,0);
    }

  */
  
  return 0;
}

void CGoughArt::Fermer()
{
  //comedi_close(mDevice);
}


// Commande articulaire en position
int CGoughArt::AllerEnPositionArticulaireAbsolue(double q[NB_VERINS])
{

  for (unsigned int i=0; i<NB_VERINS; i++)
    {
      mConsigne[i]=q[i];
      int tmp=(int)(q[i]*mParam[i].PasMotParMetre
		    +mParam[i].Lmin);
      if ( tmp >= mParam[i].Lmax)		// Butée logicielle supérieure
	{
	  tmp = mParam[i].Lmax;
	}
      if ( tmp <= mParam[i].Lmin )		// Butée logicielle inférieure
	{
	  tmp = mParam[i].Lmin ;
	}
      mCommandes[i]=tmp;

    }




  /*
  int ret = comedi_do_insnlist(mDevice,&str_list_Ecriture);
	//Execution des tâches

  if (ret<0)
    {	
      cout << "Erreur: " << ret<< endl;
      comedi_perror(DEVICE_COMEDI);
      return -1;
    }
  */

  return 0;
}

// Increment de position articulaire
int CGoughArt::IncrementerPositionArticulaire(double deltaq[NB_VERINS])
{
  //cout << "IncrementerPositionArticulaire non disponible"<< endl;;
  double q[NB_VERINS];

  int ret=LireCodeurs(q);
  if (ret<0)
    {
      cout << "Erreur: " << ret<< endl;
      //comedi_perror(DEVICE_COMEDI);
      return -1;
    }

  for (int i=0; i<NB_VERINS; i++)
    {
      q[i]+=deltaq[i];
    };

  ret=AllerEnPositionArticulaireAbsolue(q);
  if (ret<0)
    {
      cout << "Erreur: " << ret<< endl;
      //comedi_perror(DEVICE_COMEDI);
      return -1;
    }

  return 0;
}

#define ERREUR_MAX 5e-4
#define NB_ITER_MAX 30000

int CGoughArt::AttendreFinMvt()
{
  int fin=false;
  double erreur, tmp;
  double q_lu[NB_VERINS];

  int nb_iter=0;
  while (!fin)
    {
      nb_iter++;
      erreur=0.0;
      int ret=LireCodeurs(q_lu);
      if (ret<0)
	{
	  cout << "Erreur: " << ret<< endl;
	  //comedi_perror(DEVICE_COMEDI);
	  return -1;
	}
      for (int i=0; i<NB_VERINS; i++)
	{
	  tmp=fabs(mConsigne[i]-q_lu[i]);
	  erreur=max(erreur,tmp);
	}
      fin=((erreur<ERREUR_MAX)||(nb_iter>NB_ITER_MAX));
      //cout << "Erreur : " << erreur << endl;
      if (nb_iter>NB_ITER_MAX)
	{
	  cout << "NB_ITER_MAX atteint" << endl;
	}
    }
  // On attend encore 1 seconde pour etre sur de la convergence
  usleep(1000000);
  return 0;
}


//************************************************************************************************************************************************
// Commande en vitesse articulaire (en m/s)

void CGoughArt::ArreterVitesseArticulaire()
{
  mArretVitesse=true;
}


void *LancerBoucleVitesse(void* param)
{
  cout << "Ici" << endl;
	// appeler la fonction BoucleVitesse()
	CGoughArt *robot=reinterpret_cast<CGoughArt *>(param);

	robot->BoucleVitesse();
	pthread_exit(NULL); 
} 

int CGoughArt::EnvoyerVitesseArticulaire(double qpoint[NB_VERINS])
{

  for (int i=0; i<NB_VERINS; i++)
    {
      if (qpoint[i]>0)
	{
	  mVitesse[i]=qpoint[i]+FS;
	}
      else if (qpoint[i]<0)
	{
	  mVitesse[i]=qpoint[i]-FS;
	}
      else
	{
	  mVitesse[i]=qpoint[i];
	}
    }
     

     if (mArretVitesse)
     { 
        
       mArretVitesse=false;
	//La creation et l'activation d'une thread
	pthread_create(&mBoucleVitesse,NULL,LancerBoucleVitesse,this);
     }
  return 0;
}







void CGoughArt::BoucleVitesse()
{
  double deltaq[NB_VERINS];

  int cpt=0;
  cerr << "Lancement" << endl;

  double temps=PeriodeEchantillonnage/1000000.;
  
  cerr << "temps : " << temps << endl;


  while (!mArretVitesse)
    {
      cpt++;
      for (int i=0; i<NB_VERINS; i++)
	{
	  //lecture de mVitesse
	  deltaq[i]=mVitesse[i]*temps;		
	  //incrementer la position articulaire en ajoutant deltaq
	  
	}
      IncrementerPositionArticulaire(deltaq);
      
      usleep(PeriodeEchantillonnage);
      
    }


	
}





//*****************************************************************************************************************************************************
// Lecture des positions articulaires
int CGoughArt::LireCodeurs(double q[NB_VERINS])
{
 
  /*
  int ret=comedi_do_insnlist(mDevice,&str_list_Lecture);
  if  (ret<0)
    {
      cout << "  Erreur : " << ret<< endl;
      comedi_perror(DEVICE_COMEDI);
      return -1;
    }
  
  */

  double tensions[NB_VERINS];
  int error=LireTensionsCodeurs(tensions);

  for(int i=0;i<NB_VERINS;i++)
    {
      // Conversion
      q[i]=(tensions[i]-mParam[i].ZeroCodeur)/mParam[i].PasCodParMetre;

    }
  
  return error;
}


TButee CGoughArt::EstEnButee(double q, int num_verin)
{
  if (q>mParam[num_verin].LongueurMax)
    {
      return QMAX;
    }
  else if (q<mParam[num_verin].LongueurMin)
    {
      return QMIN ;
    }
  else
    {
      return QOK;
    };

}

// Lecture des positions articulaires en unites codeurs
int CGoughArt::LireTensionsCodeurs(double V[NB_VERINS])
{
  
  /*
  int ret=comedi_do_insnlist(mDevice,&str_list_Lecture);
  if  (ret<0)
    {
      cout << "  Erreur : " << ret<< endl;
      comedi_perror(DEVICE_COMEDI);
      return -1;
    }
  */

		unsigned int arg[(16*3) + 1 + 1] = {0};
  		int error = 0;
		unsigned int i = 0;

		/* Number of channel to read (1 - 16) */
		arg[0] = 16;

		/* List of channels to read, here channel 0 to 15 */
		for (i=0; i<arg[0]; i++)
			arg[(1+i)] = i; //entree_carte[i];

		/* Set the gain for each channel */
		for (i=0; i<arg[0]; i++)
			arg[(arg[0] + 1 + i)] = APCI3120_1_GAIN;

		/* Set the polarity for each channel : Polarity can be APCI3120_UNIPOLAR or APCI3120_BIPOLAR 				*/
		for (i=0; i<arg[0]; i++)
			arg[(arg[0] + arg[0] + 1 + i)] = APCI3120_BIPOLAR;

		/* Set the converting time (it is the same for each channel) it can be of 10 up to 32767 us*/
		arg[(arg[0] + arg[0] + arg[0] + 1)] = 10;

		error = ioctl (mCarteAPCI3120[0], CMD_APCI3120_ReadMoreAnalogInputs, arg);

		switch (error)
		{			
			case 0: 
				for (i=0; i<16; i++)
					printf ("Channel %i = %u\n", i, arg[i]);
				for (i=0; i<NB_VERINS; i++)
					printf ("Lecture %i = %u\n", i, arg[entree_carte[i]]);
			break;

			case 1: printf ("The handle parameter of the board is wrong.\n");
			break;

			case 2: printf ("The size of the sequence table is wrong.\n");
			break;

			case 3: printf ("Wrong parameter found in table the channel array\n");
			break;

			case 4: printf ("Wrong parameter found in table the gain array\n");
			break;

			case 5: printf ("Wrong parameter found in table the polarity array\n");
			break;

			case 6: printf ("he selected conversion time is wrong.\n");
			break;

			case 7: /* Can not occured */
			break;

			case 8: printf ("Time out.\n");
			break;

			default:perror("ioctl");
			break;
		}

  for(i=0;i<NB_VERINS;i++)
    {
      V[i]=(int)arg[i];

    }
  
  return error;
}
 
// Commandes en tensions (en unites codeur)
int CGoughArt::EnvoyerTensionsCodeurs(double V[NB_VERINS])
{


  		unsigned int arg[(8*2) + 1 + 1] = {0};
		int error = 0;
		unsigned int i = 0;

		/* First channel to write */
		arg[0] = 0;

		/* Number of channels to write */
		arg[1] = 8;

		/* Set the polarity for each channel */
		for (i=0; i<arg[1]; i++)
			arg[(2 + i)] = APCI3120_BIPOLAR; /* Polarity can be APCI3120_UNIPOLAR or APCI3120_BIPOLAR 				*/

		/* Set the value to write on each channel : Value to write (Unipolar 0 - 8192, Bipolar 0 - 16383) 				*/
		for (i=0; i<arg[1]; i++)
		{ 
			if (V[i]<5000) 
				arg[(2 + i + 8)] = 5000;
			else if (V[i]> 12000)
				arg[(2 + i + 8)] = 12000;
			else
				arg[(2 + i + 8)] = V[i];
		}

		error = ioctl (mCarteAPCI3120[0], CMD_APCI3120_WriteMoreAnalogOutput, arg);

		switch (error)
		{			
			case 0: 
				for (i=0; i<arg[1]; i++)
					printf ("Write %u on channel %i\n", arg[(2 + i + 8)], (i + arg[0]));
			break;

			case 1: printf ("The handle parameter of the board is wrong.\n");
			break;

			case 2: printf ("Number of the analog output is wrong.\n");
			break;

			case 3: printf ("The selected output voltage range is wrong.\n");
			break;

			case 4: printf ("Wrong output value for the output channel.\n");
			break;

			case 5: printf ("Time out.\n");
			break;

			default:perror("ioctl");
			break;
		}
 
   return error;
}

