/***********************************************************

                     Classe CGoughArt

  Contient les methodes pour la commande articulaire
  d'une plateforme de Gough-Stewart 'symetrique'

  Auteur : N. Andreff, LASMEA, Clermont-Ferrand, France
  Date :   12 juillet 2004


  Conventions :
    - Les variables articulaires sont comprises entre 0 et
      l'allongement maximal du verin et exprimees en metres

************************************************************/


#ifndef _CGOUGH_ART_HH
#define _CGOUGH_ART_HH
#include <pthread.h>


#define NB_VERINS 6
#include "CGoughParamArt.hh"

enum  EButee {QMAX, QMIN, QOK}; 
typedef EButee TButee;

class CGoughArt 
{
public:
  // Constructeur par defaut
  CGoughArt();
  // Constructeur pour NB_VERINS jambes identiques
  CGoughArt(const CGoughParamArt& Param);
  // Constructeur pour NB_VERINS jambes differentes
  CGoughArt(const CGoughParamArt Param[NB_VERINS]);

  // Ouvre la communication avec la carte d'E/S
  int Ouvrir();
  // Ferme la communication avec la carte d'E/S
  void Fermer();

  // Commande articulaire en position (en m)
  int AllerEnPositionArticulaireAbsolue(double q[NB_VERINS]);
  // Increment de position articulaire (en m)
  int IncrementerPositionArticulaire(double deltaq[NB_VERINS]);
  // Attente de fin de mouvement
  int AttendreFinMvt();
  // Commandes en tensions (en unites codeur)
  int EnvoyerTensionsCodeurs(double V[NB_VERINS]);

  // Commande en vitesse articulaire (en m/s)
  int EnvoyerVitesseArticulaire(double qpoint[NB_VERINS]);
  void ArreterVitesseArticulaire();

  // Lecture des positions articulaires (en m)
  int LireCodeurs(double q[NB_VERINS]);
  // Renvoie 
  TButee EstEnButee(double q, int num_verin);
  // Lecture des positions articulaires (en unites codeur)
  int LireTensionsCodeurs(double V[NB_VERINS]);

private:

  // L'acces logique a la carte d'E/S
  int * mCarteAPCI3120;
  int entree_carte[6], sortie_carte[6]; // representation logique du cablage


  // Les structures de donnees remanentes pour l'envoi de commandes
  float mCommandes[NB_VERINS];


  // Les structures de donnees remanentes pour la lecture de donnees
  float mLectureCodeurs[NB_VERINS];

  // Les parametres des verins
  CGoughParamArt mParam[NB_VERINS];

  // Pour tester la fin du mouvement
  double mConsigne[NB_VERINS];


// ******************************************************************************************************************************** 
  // Pour la commande en vitesse
  bool mArretVitesse;

   double mVitesse[NB_VERINS];
   pthread_t mBoucleVitesse;
   void BoucleVitesse();
   friend void *LancerBoucleVitesse(void* param);


   
//**********************************************************************************************************************************  
};


#endif
