/***********************************************************

                     Classe CGoughParamArt

  Contient les parametres decrivant l'actionnement d'un verin 
    d'une plateforme de Gough-Stewart

  Auteur : N. Andreff, LASMEA, Clermont-Ferrand, France
  Date :   12 juillet 2004


  Conventions :
    - Les variables articulaires sont comprises entre 0 et
      l'allongement maximal du verin et exprimees en metres

************************************************************/


#ifndef _CGOUGH_PARAM_ART_HH
#define _CGOUGH_PARAM_ART_HH

class CGoughParamArt {
public:
  // Longueurs caracteristiques de la course d'un verin (en m)
  double LongueurMin, LongueurMax;

  // Les memes exprimees en unites moteur (PasMot)
  int Lmin, Lmax;
  double PasMotParMetre;

  // Conversion metres vers unites codeur (PasCod)
  int ZeroCodeur;
  double PasCodParMetre;
  

  // Constructeur avec initialisation par defaut aux
  // valeurs CAO de la plateforme DeltaLab
  CGoughParamArt()
  {
    LongueurMin=0.345; LongueurMax=0.485;
    Lmin = 4510; Lmax = 12000;
    ZeroCodeur = 3340;
    PasCodParMetre = 427786;
    PasMotParMetre = (Lmax-Lmin)/(LongueurMax-LongueurMin);
  };

};


#endif
