#include "CVision.hh"

pthread_mutex_t gMutexSuivi = PTHREAD_MUTEX_INITIALIZER;
pthread_t gBoucleSuivi;

CVision::CVision()
  : mInit(false), m_list_d(NULL)
{

}


CVision::~CVision()
{
}


int CVision::InitCamera()
{
  try {
    mCamera.open(mImage) ;
  }
  catch(...) {
    std::cout << "Impossible de detecter la camera" << std::endl;
    return -1;
  }
  
  try {
    mCamera.setFramerate(34);
    //g.setNBuffers(1);
  }
  catch(...) {
    vpERROR_TRACE("Mauvaise frequence");
    return -2;
  }

  try {
    mCamera.acquire(mImage);
  }
  catch(...) {
    vpERROR_TRACE(" Error caught during acquiring");
    return -3;
  }

  try{
    // Display size is automatically defined by the image (mImage) size
    display.init(mImage, 100, 100,"Display...") ;
    // Display the image
    // The image class has a member that specify a pointer toward
    // the display that has been initialized in the display declaration
    // therefore is is no longuer necessary to make a reference to the
    // display variable.
    vpDisplay::display(mImage) ;
    vpDisplay::flush(mImage) ;
  }
  catch(...)
    {
      vpERROR_TRACE("Error while displaying the image") ;
      return -4;
    }

  mInit=true;
  return 0;
}


void CVision::ArreterSuivi()
{
  mArretSuivi=true;

  pthread_join(gBoucleSuivi, NULL);
}

void *LancerBoucleSuivi ( void* param )
{
  // appeler la fonction BoucleVitesse()
  CVision *camera=reinterpret_cast<CVision *> ( param );

  int ret=camera->BoucleSuivi();

  if (ret)
    {
      vpERROR_TRACE(" Erreur durant la boucle de suivi");
    }
  pthread_exit ( NULL );
}

int CVision::BoucleSuivi()
{


  while (!mArretSuivi)
    {
      try {
	mCamera.acquire(mImage);
      }
      catch(...) {
	vpERROR_TRACE(" Error caught during acquiring");
	return -3;
      }
      
      try{
	vpDisplay::display(mImage) ;
	//vpDisplay::flush(mImage) ;
      }
      catch(...)
	{
	  vpERROR_TRACE("Error while displaying the image") ;
	  return -4;
	}
      
      //mutex
      pthread_mutex_lock( &gMutexSuivi );
      
      if (m_list_d)
	{
	  m_list_d->kill();
	  delete m_list_d;
	}
      
      m_list_d = d.searchDotsInArea(mImage, 0, 0, 
				    mImage.getWidth(), 
				    mImage.getHeight()) ;
      
      if( m_list_d->nbElement() == 0 ) {
	std::cout << "Dot auto detection did not work."
		  << std::endl;
	exit(-1) ;
      }
      else {
	std::cout << std::endl << m_list_d->nbElement() << " dots are detected" << std::endl;
	
	//if (opt_display) {
	if (1) {
	  int i=0;
	  // Parse all founded dots for display
	  m_list_d->front();
	  while (!m_list_d->outside()) {
	    vpDot2 tmp_d;
	    tmp_d = m_list_d->value() ;
	    
	    std::cout << "Dot " << i++ << " : " << tmp_d.get_u()
		      << " " << tmp_d.get_v() << std::endl;
	
	    m_list_d->next() ;
	    vpDisplay::displayCross_uv(mImage,
				       (int)tmp_d.get_u(), 
				       (int)tmp_d.get_v(),
				       10, vpColor::red) ;
	  }
	  vpDisplay::flush(mImage) ;
	}
      }
     
      pthread_mutex_unlock( &gMutexSuivi );
      
    }

  return 0;
}

int CVision::LancerSuivi()
{

  if (!mInit)
    {
      vpERROR_TRACE(" Camera non initialisee.");
      vpERROR_TRACE(" Appel de CVision::InitCamera() necessaire");
      return -1;
    }

  /*  unsigned opt_first = 1;
  unsigned opt_nimages = 10;
  unsigned opt_step = 1;*/
  double opt_sizePrecision = 0.65;
  double opt_grayLevelPrecision = 0.85;
  double opt_ellipsoidShapePrecision = 0.8;

  d.setGraphics(true);
  try{
    d.setGrayLevelPrecision(opt_grayLevelPrecision);
    
    std::cout << "Please click on a dot to initialize detection"
	      << std::endl;
    
    d.initTracking(mImage) ;
    //if (opt_display) {
    if (1) {
      vpDisplay::displayCross_uv(mImage,
				 (int)d.get_u(),
				 (int)d.get_v(),
				 10,vpColor::green) ;
      vpDisplay::flush(mImage) ;
    }
    d.setSizePrecision(opt_sizePrecision);
    d.setEllipsoidShapePrecision(opt_ellipsoidShapePrecision);
    printf("Dot characteristics: \n");
    printf("  width : %lf\n", d.getWidth());
    printf("  height: %lf\n", d.getHeight());
    printf("  surface: %lf\n", d.getSurface());
    printf("  gray level min: %d\n", d.getGrayLevelMin());
    printf("  gray level max: %d\n", d.getGrayLevelMax());
    printf("  grayLevelPrecision: %lf\n", 
	   d.getGrayLevelPrecision());
    printf("  sizePrecision: %lf\n", 
	   d.getSizePrecision());
    printf("  ellipsoidShapePrecision: %lf\n", 
	   d.getEllipsoidShapePrecision());
  }
  catch(...)
    {
      std::cerr << "Cannot initialize the tracking..."
		<< std::endl;
      return -1;
    }
  


  if (mArretSuivi)
    { 
      
      mArretSuivi=false;
      
      std::cerr << "ok ? ... "; 
      //La creation et l'activation d'une thread
      pthread_create(&gBoucleSuivi,NULL,LancerBoucleSuivi,this);

      std::cerr << "ok" << std::endl;
    }
  
  return 0;
}



vpList<vpDot2>* CVision::Points()
{

  vpList<vpDot2>* Liste=new vpList<vpDot2>();

  pthread_mutex_lock( &gMutexSuivi );
  while (!m_list_d->outside()) {
    vpDot2 tmp_d;
    tmp_d = m_list_d->value() ;

    Liste->addRight(tmp_d);

    m_list_d->next() ;
  }
  pthread_mutex_unlock( &gMutexSuivi );

  return Liste;
}
