//------------------------------------------------------------------------------
/** Filter the "." and ".." entries in the struct dirent ** area set up by the scandir() function (utility function).*/
int
get_real_dir_entry (const struct dirent * ent)
{
        if (! strcmp(ent->d_name,"."))
                return 0;
        if (! strcmp(ent->d_name,".."))
                return 0;
                                
        return 1;
}

//--------------------------------------------------------------------------------
/** this function look up for cards in the /proc/sys/apci3120/ directory 
* @param cards A pointer to the card area to fill
* @return The number of cards detected (may be zero if nor cards found).
*/
int apci3120_find_cards(int ** cards)
{
	
	struct dirent **eps;
	int cnt = 0; // number of card found
	char APCI_SPECIAL_FILE_DIR[255]={0}; 
	char proc_path_template[255]={0};

	/* Test which node is availabe */
	{
		struct stat file_stat;

		if (stat("/proc/sys/apci3120",&file_stat) == 0)
		{
			/* Kernel 2.4 */
			strncpy(APCI_SPECIAL_FILE_DIR, "/proc/sys/apci3120/", 19);
			strncpy(proc_path_template, "%s%s/ctrl", 9);
		}
		else if (stat("/dev/apci3120",&file_stat) == 0)
		{
			/* Kernel 2.6 */
			strncpy(APCI_SPECIAL_FILE_DIR, "/dev/apci3120/", 14);
			strncpy(proc_path_template, "%s%s", 4);
		}
		else
		{
			perror("fstat");
			exit(1);
		}
	}
	
	/* scan directory for card entry */
	if ( (cnt = scandir (APCI_SPECIAL_FILE_DIR, &eps, get_real_dir_entry , alphasort)) == -1 )
	{
		perror("scandir");
		return 0;
	}	
	
	if (cnt == 0) return 0;
	
	/* allocate cards area */
	if ( ! ( (*cards) = (int *) malloc( cnt * sizeof(int) ) ) )
	{
		perror("malloc");
		free(eps);
		return 0;
	}
	
	/* for each card entry, open it and store file descriptor in cards[] */
	{
		int i;
		for (i=0; i< cnt; i++)
		{
			int fd;
			char name[100];
			
			snprintf(name,sizeof(name),proc_path_template,APCI_SPECIAL_FILE_DIR,eps[i]->d_name);
			
			if ( ( fd = open(name, O_RDWR)) == -1 )
			{
				perror("open");
				exit(1);
			}			
									
			{
				struct stat file_stat;
				if ( fstat(fd,&file_stat) )
				{
					perror("fstat");
					exit(1);
				}			
				if (!S_ISCHR(file_stat.st_mode)) 
				{
					/* should never happen !*/
					printf("file %d is not a character device\n",i);
					exit(1);
				}
				printf("%s, major:%d minor:%d \n",name, major(file_stat.st_rdev),minor(file_stat.st_rdev));
			
				(*cards)[minor(file_stat.st_rdev)] = fd;
			} 
			/* index device file descriptor by the minor number */			
		}	
	}
		
	return cnt;
}
